@@ -93,7 +93,7 @@ using namespace llvm::sys;
 using namespace clang;
 using namespace clang::driver;
 using namespace clang::driver::options;
-using namespace TimeStatistics;
+using namespace COMGR::TimeStatistics;
 
 namespace COMGR {
 
@@ -205,10 +205,10 @@ bool AssemblerInvocation::createFromArgs(AssemblerInvocation &Opts,
   // Parse the arguments.
   const OptTable &OptTbl = getDriverOptTable();
 
-  const unsigned IncludedFlagsBitmask = options::CC1AsOption;
+  llvm::opt::Visibility VisibilityMask(options::CC1AsOption);
   unsigned MissingArgIndex, MissingArgCount;
   InputArgList Args = OptTbl.ParseArgs(Argv, MissingArgIndex, MissingArgCount,
-                                       IncludedFlagsBitmask);
+                                       VisibilityMask);
 
   // Check for missing argument error.
   if (MissingArgCount) {
@@ -221,7 +221,7 @@ bool AssemblerInvocation::createFromArgs(AssemblerInvocation &Opts,
   for (const Arg *A : Args.filtered(OPT_UNKNOWN)) {
     auto ArgString = A->getAsString(Args);
     std::string Nearest;
-    if (OptTbl.findNearest(ArgString, Nearest, IncludedFlagsBitmask) > 1) {
+    if (OptTbl.findNearest(ArgString, Nearest, VisibilityMask) > 1) {
       Diags.Report(diag::err_drv_unknown_argument) << ArgString;
     } else {
       Diags.Report(diag::err_drv_unknown_argument_with_suggestion)
@@ -668,7 +668,8 @@ AMDGPUCompiler::executeInProcessDriver(ArrayRef<const char *> Args) {
   DiagnosticsEngine Diags(DiagID, &*DiagOpts, DiagClient);
   ProcessWarningOptions(Diags, *DiagOpts, /*ReportDiags=*/false);
 
-  Driver TheDriver((Twine(env::getLLVMPath()) + "/bin/clang").str(), "", Diags);
+  Driver TheDriver((Twine(env::getLLVMPath()) + "/bin/clang").str(),
+                   llvm::sys::getDefaultTargetTriple(), Diags);
   TheDriver.setTitle("AMDGPU Code Object Manager");
   TheDriver.setCheckInputsExist(false);
 
@@ -949,6 +950,8 @@ AMDGPUCompiler::processFiles(amd_comgr_data_kind_t OutputKind,
     if (auto Status = amd_comgr_create_data(OutputKind, &OutputT)) {
       return Status;
     }
+
+    // OutputT can be released after addition to the data_set
     ScopedDataObjectReleaser SDOR(OutputT);
 
     DataObject *Output = DataObject::convert(OutputT);
@@ -1032,6 +1035,9 @@ amd_comgr_status_t AMDGPUCompiler::addCompilationFlags() {
   // HIP headers depend on hsa.h which is in ROCM_DIR/include.
   ROCMIncludePath = (Twine(env::getROCMPath()) + "/include").str();
 
+  // Default to O3 for all contexts
+  Args.push_back("-O3");
+
   Args.push_back("-x");
 
   switch (ActionInfo->Language) {
@@ -1050,10 +1056,8 @@ amd_comgr_status_t AMDGPUCompiler::addCompilationFlags() {
     break;
   case AMD_COMGR_LANGUAGE_HIP:
     Args.push_back("hip");
-    Args.push_back("-std=c++11");
-    Args.push_back("-target");
-    Args.push_back("x86_64-unknown-linux-gnu");
-    Args.push_back("--cuda-device-only");
+    Args.push_back("-nogpuinc");
+    Args.push_back("--offload-device-only");
     Args.push_back("-isystem");
     Args.push_back(ROCMIncludePath.c_str());
     Args.push_back("-isystem");
@@ -1074,8 +1078,6 @@ amd_comgr_status_t AMDGPUCompiler::addDeviceLibraries() {
     return AMD_COMGR_STATUS_ERROR;
   }
   Args.push_back(Saver.save(Twine("--rocm-path=") + FakeRocmDir).data());
-  Args.push_back("-mllvm");
-  Args.push_back("-relink-builtin-bitcode-postop");
   NoGpuLib = false;
 
   for (auto DeviceLib : getDeviceLibraries()) {
@@ -1143,6 +1145,13 @@ amd_comgr_status_t AMDGPUCompiler::compileToBitcode(bool WithDeviceLibs) {
     if (auto Status = addDeviceLibraries()) {
       return Status;
     }
+
+    // Currently linking postopt is only needed for OpenCL. If this becomes
+    // necessary for HIP (for example if HIP adopts the same AMDGPUSimplifyLibs
+    // strategy that potentially introduces undefined device-library symbols),
+    // we will need also apply this option in compileToRelocatable().
+    Args.push_back("-Xclang");
+    Args.push_back("-mlink-builtin-bitcode-postopt");
   }
 
   return processFiles(AMD_COMGR_DATA_KIND_BC, ".bc");
@@ -1234,6 +1243,145 @@ amd_comgr_status_t AMDGPUCompiler::compileToFatBin() {
   return Status;
 }
 
+amd_comgr_status_t AMDGPUCompiler::unbundle() {
+  if (auto Status = createTmpDirs()) {
+    return Status;
+  }
+
+  // Collect bitcode memory buffers from bitcodes, bundles, and archives
+  for (auto *Input : InSet->DataObjects) {
+
+    std::string FileExtension;
+    amd_comgr_data_kind_t UnbundledDataKind;
+    switch(Input->DataKind) {
+    case AMD_COMGR_DATA_KIND_BC_BUNDLE:
+      FileExtension = "bc";
+      UnbundledDataKind = AMD_COMGR_DATA_KIND_BC;
+      break;
+    case AMD_COMGR_DATA_KIND_AR_BUNDLE:
+      FileExtension = "a";
+      UnbundledDataKind = AMD_COMGR_DATA_KIND_AR;
+      break;
+    case AMD_COMGR_DATA_KIND_OBJ_BUNDLE:
+      FileExtension = "o";
+      UnbundledDataKind = AMD_COMGR_DATA_KIND_EXECUTABLE;
+      break;
+    default:
+      return AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT;
+    }
+
+    // Configure Offload Bundler
+    OffloadBundlerConfig BundlerConfig;
+    BundlerConfig.AllowMissingBundles = true;
+    BundlerConfig.FilesType = FileExtension;
+    BundlerConfig.HipOpenmpCompatible = 1;
+    BundlerConfig.AllowNoHost = 1;
+
+    // Generate random name if none provided
+    if (!strcmp(Input->Name, "")) {
+      char *buf = (char *) malloc(sizeof(char) * 30);
+      sprintf(buf,"comgr-bundle-%d.%s", std::rand() % 10000,
+              FileExtension.c_str());
+      Input->Name = buf;
+    }
+
+    // Write input file system so that OffloadBundler API can process
+    // TODO: Switch write to VFS
+    std::string input_file_path = getFilePath(Input, InputDir).str().str();
+    if (auto Status = outputToFile(Input, input_file_path)) {
+      return Status;
+    }
+
+    // Bundler input name
+    BundlerConfig.InputFileNames.push_back(input_file_path);
+
+    // Generate prefix for output files
+    std::string output_prefix = std::string(Input->Name);
+    size_t index = output_prefix.find_last_of(".");
+    output_prefix = output_prefix.substr(0, index);
+
+    // Bundler target and output names
+    for (auto entry: ActionInfo->BundleEntryIDs) {
+      BundlerConfig.TargetNames.push_back(entry);
+
+      // Add an output file for each target
+      std::string output_file_name = output_prefix + '-' + entry + "." +
+        FileExtension;
+
+      // TODO: Switch this to LLVM path APIs
+      std::string output_file_path = OutputDir.str().str() + "/" +
+        output_file_name;
+      BundlerConfig.OutputFileNames.push_back(output_file_path);
+    }
+
+    OffloadBundler Bundler(BundlerConfig);
+
+    // TODO: log vectors, build clang command
+    if (env::shouldEmitVerboseLogs()) {
+      LogS << "Extracting Bundle:\n"
+        << "\t  Unbundled Files Extension: ." << FileExtension << "\n"
+        << "\t  Bundle Entry ID: " << BundlerConfig.TargetNames[0] << "\n"
+        << "\t   Input Filename: " << BundlerConfig.InputFileNames[0] << "\n"
+        << "\t  Output Filename: " << BundlerConfig.OutputFileNames[0]
+        << "\n";
+      LogS.flush();
+    }
+
+    switch(Input->DataKind) {
+    case AMD_COMGR_DATA_KIND_BC_BUNDLE: {
+      llvm::Error Err = Bundler.UnbundleFiles();
+      llvm::logAllUnhandledErrors(std::move(Err), llvm::errs(),
+                                  "Unbundle Bitcodes Error: ");
+      break; }
+    case AMD_COMGR_DATA_KIND_AR_BUNDLE: {
+      llvm::Error Err = Bundler.UnbundleArchive();
+      llvm::logAllUnhandledErrors(std::move(Err), llvm::errs(),
+                                  "Unbundle Archives Error: ");
+      break; }
+    case AMD_COMGR_DATA_KIND_OBJ_BUNDLE: {
+      llvm::Error Err = Bundler.UnbundleFiles();
+      llvm::logAllUnhandledErrors(std::move(Err), llvm::errs(),
+                                  "Unbundle Objects Error: ");
+      break; }
+    default:
+      llvm_unreachable("invalid bundle type");
+    }
+
+    // Add new bitcodes to OutSetT
+    for (auto output_file_path : BundlerConfig.OutputFileNames) {
+
+      amd_comgr_data_t ResultT;
+
+      if (auto Status = amd_comgr_create_data(UnbundledDataKind,
+                                              &ResultT))
+        return Status;
+
+      // ResultT can be released after addition to the data_set
+      ScopedDataObjectReleaser SDOR(ResultT);
+
+      DataObject *Result = DataObject::convert(ResultT);
+      if (auto Status = inputFromFile(Result, StringRef(output_file_path)))
+        return Status;
+
+      StringRef output_file_name =
+        llvm::sys::path::filename(StringRef(output_file_path));
+      Result->setName(output_file_name);
+
+      if (auto Status = amd_comgr_data_set_add(OutSetT, ResultT)) {
+        return Status;
+      }
+
+      // Remove input and output file after reading back into Comgr data
+      if (!env::shouldEmitVerboseLogs()) {
+        sys::fs::remove(input_file_path);
+        sys::fs::remove(output_file_path);
+      }
+    }
+  }
+
+  return AMD_COMGR_STATUS_SUCCESS;
+}
+
 amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
   if (auto Status = createTmpDirs()) {
     return Status;
@@ -1256,7 +1404,6 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
       // string to assign. This string is used when the DataObject is written
       // to the file system via SAVE_TEMPS, or if the object is a bundle which
       // also needs a file system write for unpacking
-
       char *buf = (char *) malloc(sizeof(char) * 30);
       sprintf(buf,"comgr-anon-bitcode-%d.bc", std::rand() % 10000);
 
@@ -1270,15 +1417,14 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
     }
 
     if (Input->DataKind == AMD_COMGR_DATA_KIND_BC) {
-      // The data in Input outlives Mod, and the linker destructs Mod after
-      // linking it into composite (i.e. ownership is not transferred to the
-      // composite) so MemoryBuffer::getMemBuffer is sufficient.
-
       if (env::shouldEmitVerboseLogs()) {
         LogS << "\t     Linking Bitcode: " << InputDir << "/" << Input->Name
              << "\n";
       }
 
+      // The data in Input outlives Mod, and the linker destructs Mod after
+      // linking it into composite (i.e. ownership is not transferred to the
+      // composite) so MemoryBuffer::getMemBuffer is sufficient.
       auto Mod =
         getLazyIRModule(MemoryBuffer::getMemBuffer(
             StringRef(Input->Data, Input->Size), "", false),
@@ -1300,6 +1446,8 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
       }
 
       // Determine desired bundle entry ID
+      // TODO: Move away from using ActionInfo->IsaName
+      //   Use ActionInfo->BundleEntryIDs instead
       if (!ActionInfo->IsaName)
         return AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT;
 
@@ -1357,7 +1505,6 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
         LogS.flush();
       }
 
-
       llvm::Error Err = Bundler.UnbundleFiles();
       llvm::logAllUnhandledErrors(std::move(Err), llvm::errs(),
                                   "UnbundleFiles error: ");
@@ -1367,6 +1514,9 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
       if (auto Status = amd_comgr_create_data(AMD_COMGR_DATA_KIND_BC, &ResultT))
         return Status;
 
+      // ResultT can be released after addition to the data_set
+      ScopedDataObjectReleaser SDOR(ResultT);
+
       DataObject *Result = DataObject::convert(ResultT);
       if (auto Status = inputFromFile(Result, StringRef(output_file_path)))
         return Status;
@@ -1386,8 +1536,6 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
         return AMD_COMGR_STATUS_ERROR;
       if (L.linkInModule(std::move(Mod), ApplicableFlags))
         return AMD_COMGR_STATUS_ERROR;
-
-      Result->release();
     }
     // Unbundle bitcode archive
     else if (Input->DataKind == AMD_COMGR_DATA_KIND_AR_BUNDLE) {
@@ -1397,6 +1545,8 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
       }
 
       // Determine desired bundle entry ID
+      // TODO: Move away from using ActionInfo->IsaName
+      //   Use ActionInfo->BundleEntryIDs instead
       if (!ActionInfo->IsaName)
         return AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT;
 
@@ -1465,6 +1615,9 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
       if (auto Status = amd_comgr_create_data(AMD_COMGR_DATA_KIND_AR, &ResultT))
         return Status;
 
+      // ResultT can be released after addition to the data_set
+      ScopedDataObjectReleaser SDOR(ResultT);
+
       DataObject *Result = DataObject::convert(ResultT);
       if (auto Status = inputFromFile(Result, StringRef(output_file_path)))
         return Status;
@@ -1525,8 +1678,6 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
 
       llvm::logAllUnhandledErrors(std::move(Err), llvm::errs(),
                                   "Unpack Archives error: ");
-
-      Result->release();
     }
     else
       continue;
@@ -1546,6 +1697,8 @@ amd_comgr_status_t AMDGPUCompiler::linkBitcodeToBitcode() {
   if (auto Status = amd_comgr_create_data(AMD_COMGR_DATA_KIND_BC, &OutputT)) {
     return Status;
   }
+
+  // OutputT can be released after addition to the data_set
   ScopedDataObjectReleaser SDOR(OutputT);
 
   DataObject *Output = DataObject::convert(OutputT);
@@ -1639,6 +1792,8 @@ amd_comgr_status_t AMDGPUCompiler::linkToRelocatable() {
           amd_comgr_create_data(AMD_COMGR_DATA_KIND_RELOCATABLE, &OutputT)) {
     return Status;
   }
+
+  // OutputT can be released after addition to the data_set
   ScopedDataObjectReleaser SDOR(OutputT);
 
   DataObject *Output = DataObject::convert(OutputT);
@@ -1693,6 +1848,7 @@ amd_comgr_status_t AMDGPUCompiler::linkToExecutable() {
           amd_comgr_create_data(AMD_COMGR_DATA_KIND_EXECUTABLE, &OutputT)) {
     return Status;
   }
+  // OutputT can be released after addition to the data_set
   ScopedDataObjectReleaser SDOR(OutputT);
 
   DataObject *Output = DataObject::convert(OutputT);
